@@ -7,6 +7,9 @@ from datetime import datetime, timedelta
 import requests
 from urllib.parse import urlparse, parse_qs
 from mcpbundles_proxy.config import save_token, load_token
+import secrets
+import hashlib
+import base64
 
 
 def find_free_port(start_port=8765, max_attempts=10):
@@ -110,11 +113,26 @@ def get_or_register_client():
     return register_client()
 
 
+def generate_pkce():
+    """Generate PKCE code_verifier and code_challenge."""
+    # Generate random code_verifier (43-128 characters)
+    code_verifier = base64.urlsafe_b64encode(secrets.token_bytes(32)).decode('utf-8').rstrip('=')
+    
+    # Generate code_challenge (SHA256 hash of code_verifier)
+    challenge_bytes = hashlib.sha256(code_verifier.encode('utf-8')).digest()
+    code_challenge = base64.urlsafe_b64encode(challenge_bytes).decode('utf-8').rstrip('=')
+    
+    return code_verifier, code_challenge
+
+
 def oauth_flow():
     """Execute OAuth flow with browser and local callback server."""
     # Get or register OAuth client
     client_id, client_secret = get_or_register_client()
     
+    # Generate PKCE parameters
+    code_verifier, code_challenge = generate_pkce()
+    
     # Find available port
     port = find_free_port()
     redirect_uri = f"http://localhost:{port}/callback"
@@ -122,13 +140,15 @@ def oauth_flow():
     # Start local callback server
     server = HTTPServer(('localhost', port), OAuthCallbackHandler)
     
-    # Open browser to OAuth page
+    # Open browser to OAuth page with PKCE
     auth_url = (
         f"https://api.mcpbundles.com/o/authorize/"
         f"?client_id={client_id}"
         f"&redirect_uri={redirect_uri}"
         f"&response_type=code"
         f"&scope=read write"
+        f"&code_challenge={code_challenge}"
+        f"&code_challenge_method=S256"
     )
     
     print(f"Opening browser for authentication...")
@@ -151,7 +171,8 @@ def oauth_flow():
             "code": OAuthCallbackHandler.auth_code,
             "client_id": client_id,
             "client_secret": client_secret,
-            "redirect_uri": redirect_uri
+            "redirect_uri": redirect_uri,
+            "code_verifier": code_verifier
         }
     )
     
