@@ -6,9 +6,12 @@ import aiohttp
 import json
 import logging
 import base64
+import ssl
+import certifi
 from typing import Dict, Any, Optional
 from asyncio import Future
 from websockets.asyncio.client import ClientConnection
+from urllib.parse import quote
 
 logger = logging.getLogger(__name__)
 
@@ -48,8 +51,14 @@ class TCPTunnel:
         """Close TCP connection."""
         self.running = False
         try:
-            self.writer.close()
-            await self.writer.wait_closed()
+            try:
+                self.writer.close()
+            except Exception:
+                pass  # writer.close() is not async, but mocks might make it async
+            try:
+                await self.writer.wait_closed()
+            except Exception:
+                pass  # Ignore wait_closed errors
         except Exception as e:
             logger.error(f"Error closing tunnel {self.tunnel_id}: {e}")
         
@@ -77,7 +86,12 @@ class TunnelClient:
         
     async def connect(self):
         """Connect and maintain WebSocket tunnel with auto-reconnect."""
-        uri = f"wss://api.mcpbundles.com/v1/tunnel/connect?token={self.token}"
+        # URL-encode the token to handle special characters
+        encoded_token = quote(self.token, safe='')
+        uri = f"wss://api.mcpbundles.com/api/tunnel/connect?token={encoded_token}"
+        
+        # Create SSL context with certifi's CA bundle
+        ssl_context = ssl.create_default_context(cafile=certifi.where())
         
         while self.running:
             try:
@@ -85,12 +99,25 @@ class TunnelClient:
                 async with websockets.connect(
                     uri,
                     ping_interval=20,
-                    ping_timeout=10
+                    ping_timeout=10,
+                    ssl=ssl_context
                 ) as ws:
                     self.ws = ws
                     self.retry_count = 0  # Reset on successful connect
                     logger.info("✅ Tunnel connected")
                     
+                    # Update status to connected
+                    try:
+                        from mcpbundles_proxy.config import save_status, STATUS_FILE
+                        if STATUS_FILE.exists():
+                            import json
+                            with open(STATUS_FILE, 'r') as f:
+                                status = json.load(f)
+                            status['tunnel_status'] = 'connected'
+                            save_status(status)
+                    except Exception:
+                        pass  # Don't fail connection if status update fails
+                    
                     await self.listen()
                     
             except websockets.exceptions.WebSocketException as e:
